#!/bin/bash
# vim:sw=8:noet

BBL=2E54B353-1271-4842-806F-E436D6AF6985
FSBL=5B193300-FC78-40CD-8002-E86C45580B47

soc_init() {
	MEDIATABLE=gpt
	START_SECTOR=34 # MiB
	ROOTPART_NUM=1
	CONSOLE=1
	SYSCON=ttyS0,115200n8
}

soc_write_rootfs() {
	[ -z "${IMAGE_OUT-}" ] || prepare_image
	create_disklabel
	create_partition
	[ -z "${IMAGE_OUT-}" ] || finish_partitioning
	sync_partitions
	format_partition
	mount_partition
	write_rootfs
	setup_fstab
}

soc_write_bootloader() {
	if [ "$MEDIATABLE" = gpt ]; then
		[ -n "$ROOTFS" ] || clear_riscv64_bootloader_partition

		# resize gpt table to 7 KiB (56 entries partition)
		sgdisk -S 56 "${MEDIA-}"
	fi

	[ -n "$TMPROOT" ] || mount_partition
	# Write U-Boot
	print_message "Searching U-Boot files for ${TARGET-}..."
	[ -d "${TMPROOT-}/usr/share/u-boot/${TARGET-}" ] ||
		fatal_error "U-Boot files for ${TARGET-} were not found"
	print_done
	if [ -f ${TMPROOT-}/usr/share/u-boot/${TARGET-}/u-boot.itb ]; then
		print_message "Writing sunxi-spl.bin for ${TARGET-}..."
		dd if="${TMPROOT-}/usr/share/u-boot/${TARGET-}/sunxi-spl.bin" of="${MEDIA-}" bs=8k seek=1 ||
		  fatal_error "failed to write SPL"
		sync
		print_done

		print_message "Writing U-Boot image for ${TARGET-}..."
		dd if="${TMPROOT-}/usr/share/u-boot/${TARGET-}/u-boot.itb" of="${MEDIA-}" bs=8k seek=5 ||
		  fatal_error "failed to write U-Boot"
		sync
		print_done
	else
		print_message "Writing u-boot-sunxi-with-spl.bin..."
		dd if="${TMPROOT-}/usr/share/u-boot/${TARGET-}/u-boot-sunxi-with-spl.bin" of="${MEDIA-}" bs=1024 seek=8 ||
		  fatal_error "failed to write U-Boot"
		sync
		print_done
	fi
}

soc_exit() {
	:
}
