#!/bin/bash
# vim:sw=8:noet

BBL=2E54B353-1271-4842-806F-E436D6AF6985
FSBL=5B193300-FC78-40CD-8002-E86C45580B47
SBI=$BBL

soc_init() {
	MEDIATABLE=gpt
	#FSBLPART_NUM=1
	#SBIPART_NUM=2
	ROOTPART_NUM=1
	# not video
	VNC=1
	CONSOLE=1
	# (32767 + 1)*512/2^20
	# ~32 KiB for FSBL
	# ~12 MiB for OpenSBI + U-Boot
	# + some overhead for future usage
	START_SECTOR=34 # MiB
	[ -n "$EFI" ] || BOOTFLAG=legacy_boot
	SYSCON=ttySIF0,115200n8
}

soc_write_rootfs() {
	[ -z "${IMAGE_OUT-}" ] || prepare_image
	create_disklabel
	print_message "Creating partition for FSBL and OpenSBI..."
	create_partition
	sync_partitions
	[ -z "${IMAGE_OUT-}" ] || finish_partitioning
	format_partition
	mount_partition
	write_rootfs
	setup_fstab
	unmount_partition
}

soc_write_bootloader() {
	[ -n "$ROOTFS" ] || clear_riscv64_bootloader_partition

	print_message "Creating partitions for U-Boot SPL and OpenSBI + U-Boot..."
	LASTPART_NUM=$(sgdisk -p "$MEDIA" | tail -n1 | tr -s ' ' | cut -f 2 -d ' ')
	let 'FSBLPART_NUM = LASTPART_NUM + 1'
	let 'SBIPART_NUM = LASTPART_NUM + 2'
	sgdisk -a 1 \
		--new=$FSBLPART_NUM:2048:2112 --change-name=$FSBLPART_NUM:fsbl \
		--typecode=$FSBLPART_NUM:"$FSBL" \
		--new=$SBIPART_NUM:2113:32767 --change-name=$SBIPART_NUM:opensbi-uboot \
		--typecode=$SBIPART_NUM:"$SBI" \
		"$MEDIA" ||
		fatal_error "failed to create FSBL and OpenSBI partitions"
	print_done

	sync_partitions
	mount_partition

	print_message "Writing FSBL..."
	FSBL_BLOB="$(readlink -e $TMPROOT/usr/share/fu540_boot/fsbl.bin)"
	[ -f "${FSBL_BLOB-}" ] || fatal_error "FSBL was not found"
	FSBLPART="${MEDIA}${partsuffix-}${FSBLPART_NUM}"
	[ -b "${FSBLPART-}" ] || fatal_error "${FSBLPART-} is not a block device or does not exist"
	dd if="$FSBL_BLOB" of="$FSBLPART" bs=4M status=none || fatal_error "failed to write FSBL"
	print_done

	print_message "Writing OpenSBI+U-Boot..."
	SBI_BLOB="$(readlink -e $TMPROOT/usr/share/opensbi/sifive/fu540/firmware/fw_payload.bin)"
	[ -f "${SBI_BLOB-}" ] || fatal_error "OpenSBI+U-Boot was not found"
	SBIPART="${MEDIA}${partsuffix-}${SBIPART_NUM}"
	[ -b "${SBIPART-}" ] || fatal_error "${SBIPART-} is not a block device or does not exist"
	dd if="$SBI_BLOB" of="$SBIPART" bs=4M status=none || fatal_error "failed to write SBI"
	sync
	print_done
}

soc_exit() {
	:
}
