#!/bin/bash
# vim:sw=8:noet

soc_init() {
	MEDIATABLE=msdos
	ROOTPART_NUM=1
	START_SECTOR=16 # MiB
	SYSCON=ttyS2,115200n8
}

soc_write_rootfs() {
	[ -z "${IMAGE_OUT-}" ] || prepare_image
	create_disklabel
	create_partition
	[ -z "${IMAGE_OUT-}" ] || finish_partitioning
	sync_partitions
	format_partition
	mount_partition
	write_rootfs
	setup_fstab
}

soc_write_bootloader() {
	[ -n "$TMPROOT" ] || mount_partition
	# Write U-Boot
	UBOOT=${TMPROOT-}/usr/share/u-boot/$TARGET/u-boot-rockchip.bin
	if [ -f "$UBOOT" ]; then
		print_message "Writing U-Boot..."
		dd if=$UBOOT of=$MEDIA seek=64 || fatal_error "failed to write U-boot"
		sync
		print_done
	else
		print_message "Writing SPL..."
		SPL=${TMPROOT-}/usr/share/u-boot/$TARGET/idbspl.img
		[ -f "$SPL" ] || SPL=${TMPROOT-}/usr/share/u-boot/$TARGET/idbloader.img
		[ -f "$SPL" ] || fatal_error "SPL image not found"
		dd if=$SPL of=$MEDIA seek=64 || fatal_error "failed to write SPL"
		sync
		print_done

		print_message "Writing U-Boot FIT image..."
		FIT=${TMPROOT-}/usr/share/u-boot/$TARGET/u-boot.itb
		[ -f "$FIT" ] || fatal_error "FIT image not found"
		dd if=$FIT of=$MEDIA seek=16384 ||
		  fatal_error "failed to write U-Boot FIT image"
		sync
		print_done
	fi
}

soc_exit() {
	:
}
