#!/bin/bash
# vim:sw=8:noet

BBL=2E54B353-1271-4842-806F-E436D6AF6985
FSBL=5B193300-FC78-40CD-8002-E86C45580B47

soc_init() {
	EFI=
	MEDIATABLE=gpt
	#BBLPART_NUM=1
	ROOTPART_NUM=1
	# not video
	VNC=1
	CONSOLE=1
	# (67583 + 1)*512/2^20
	START_SECTOR=34 # MiB
}

soc_write_rootfs() {
	[ -z "${IMAGE_OUT-}" ] || prepare_image
	create_disklabel
	create_partition
	sync_partitions
	format_partition
	mount_partition
	write_rootfs
	setup_fstab
}

soc_write_bootloader() {
	[ -n "$ROOTFS" ] || clear_riscv64_bootloader_partition

	print_message "Creating BBL partition ..."
	LASTPART_NUM=$(sgdisk -p "$MEDIA" | tail -n1 | tr -s ' ' | cut -f 2 -d ' ')
	let 'BBLPART_NUM = LASTPART_NUM + 1'
	print_message "Creating partition for BBL..."
	sgdisk -a "1" \
		--new="$BBLPART_NUM":2048:67583 --change-name="$BBLPART_NUM":bootloader \
		--typecode="$BBLPART_NUM":"$BBL" "$MEDIA" ||
		fatal_error "failed to create BBL partition"
	print_done

	sync_partitions
	mount_partition

	print_message "Writing BBL+Linux"
	BBLPART="${MEDIA}${partsuffix-}${BBLPART_NUM}"
	BBL_KERNEL="$(readlink -e ${TMPROOT-}/boot/vmlinuz)"
	[ -f "${BBL_KERNEL-}" ] || fatal_error "BBL+Linux were not found"
	[ -b "${BBLPART-}" ] || fatal_error "${BBLPART-} is not a block device or does not exist"
	dd if="${BBL_KERNEL-}" of="${BBLPART-}" bs=4M status=progress || fatal_error "failed to write BBL"
	sync
}

soc_exit() {
	:
}
