#!/bin/bash
# vim:sw=8:noet

soc_init() {
	EFI=
	MEDIATABLE=
	ROOTPART_NUM=1

	print_message "Checking options for ${TARGET-}..."
	[ "${TARGET-}" = "jetson-nano" -a -z "${ROOTFS-}" ] &&
		fatal_error "--target=jetson-nano requires option --rootfs="
	print_done

	command -v python2 >/dev/null 2>&1 ||
		fatal_error "Python 2 not installed!!!"
}

soc_write_rootfs() {
	JETSON_ARCHIVE="l4t-jetson-driver-package-32-1-jetson-nano"
	JETSON_PATH="/root"
	JETSON_TMP="/root/Linux_for_Tegra"
	if [ -n "$IMAGE_OUT" ]; then
		IMAGE_TEGRA="$IMAGE_OUT"
		IMAGE_SIZE="$IMAGE_SIZE"MiB
	else
		IMAGE_TEGRA=sd.img
		IMAGE_SIZE=128MiB
	fi
	rm -fr "$JETSON_TMP"
	if [ -f "$JETSON_PATH/$JETSON_ARCHIVE" ]; then
		print_message "Extract Jetson Nano driver archive..."
		${TAR% *} -xf "$JETSON_PATH/$JETSON_ARCHIVE" -C /root \
		&& print_done \
		|| (print_fail; wget_jetson_nano)
	else
		wget_jetson_nano
	fi
	print_message "Create Jetson Nano image ..."
	cd "$JETSON_TMP"
	sed -i 's/sudo//g' create-jetson-nano-sd-card-image.sh
	find . -type f -name "*.py" | xargs sed -i \
		-e '1s:^#!/usr/bin/env python:#!/usr/bin/env python2:'
	yes |./create-jetson-nano-sd-card-image.sh -o "$IMAGE_TEGRA" \
	  -s "$IMAGE_SIZE" -r 100  \
	|| fatal_error "Error: create Jetson Nano image failed!!!"
	print_done
	[ -z "${MEDIA-}" ] || prepare_media
	if [ -z "$IMAGE_OUT" ]; then
		print_message "Write Jetson Nano image ..."
		dd if="$IMAGE_TEGRA" of="$MEDIA" bs=1M \
		&& sgdisk -g "$MEDIA" \
		&& yes |parted -s "$MEDIA" resizepart 1 100% \
		|| fatal_error "Error: write Jetson Nano image failed!!!"
		print_done
	else
		losetup "$MEDIA" "$IMAGE_OUT"
	fi

	# Write rootfs
	[ -z "${IMAGE_OUT-}" ] || finish_partitioning
	sync_partitions
	format_partition
	mount_partition
	write_rootfs
	setup_fstab

	# Write firmware
	print_message "Writing Jetson Nano firmware..."
	(
	set -e
	firmware_jetson_nano
	) 
	[ $? -eq 0 ] || fatal_error "failed to write firmware"
	print_done
}

soc_write_bootloader() {
	[ -n "$TMPROOT" ] || mount_partition
}

soc_exit() {
	rm -fr "${JETSON_TMP-}"
}

firmware_jetson_nano() {
	cd "$JETSON_TMP"
	tar -xf nv_tegra/nvidia_drivers.tbz2 
	tar -xf nv_tegra/config.tbz2 
	mkdir -pv ${TMPROOT-}/etc/vulkan/icd.d
	cp -fv usr/lib/aarch64-linux-gnu/tegra/nvidia_icd.json ${TMPROOT-}/etc/vulkan/icd.d/
	mkdir -pv ${TMPROOT-}/lib/firmware/brcm
	cp -fv lib/firmware/brcm/* ${TMPROOT-}/lib/firmware/brcm/
	cp -fvr lib/firmware/tegra21x ${TMPROOT-}/lib/firmware/
	cp -fv lib/firmware/bcm4354.hcd ${TMPROOT-}/lib/firmware/
	cp -fv lib/firmware/nv-BT-Version ${TMPROOT-}/lib/firmware/
	cp -fv lib/firmware/tegra21x_xusb_firmware ${TMPROOT-}/lib/firmware/
	mkdir -pv ${TMPROOT-}/lib/firmware/gm20b
	find ${TMPROOT-}/lib/firmware/tegra21x -type f -o -type l |
		sed 's|^/.*/\(.*\)|\1|' |
		while read f; do
			ln -sf ../tegra21x/$f ${TMPROOT-}/lib/firmware/gm20b/$f
		done
	cp -fv usr/lib/aarch64-linux-gnu/tegra/lib*.so* ${TMPROOT-}/usr/lib64/
	cp -fv usr/lib/aarch64-linux-gnu/tegra-egl/lib*.so* ${TMPROOT-}/usr/lib64/
	mkdir -pv ${TMPROOT-}/usr/share/glvnd/egl_vendor.d
	cp -fv usr/lib/aarch64-linux-gnu/tegra-egl/nvidia.json ${TMPROOT-}/usr/share/glvnd/egl_vendor.d/
	mkdir -pv ${TMPROOT-}/usr/lib64/libv4l/plugins
	cp -fv usr/lib/aarch64-linux-gnu/libv4l/plugins/lib*.so ${TMPROOT-}/usr/lib64/libv4l/plugins/
	mkdir -pv ${TMPROOT-}/usr/lib64/X11/modules/drivers
	cp -fv usr/lib/xorg/modules/drivers/nvidia_drv.so  ${TMPROOT-}/usr/lib64/X11/modules/drivers/
	mkdir -pv ${TMPROOT-}/usr/lib64/X11/modules/extensions
	cp -fv usr/lib/xorg/modules/extensions/libglxserver_nvidia.so ${TMPROOT-}/usr/lib64/X11/modules/extensions/
	mkdir -pv ${TMPROOT-}/usr/share/alsa
	cp -fvr usr/share/alsa/* ${TMPROOT-}/usr/share/alsa/
	mkdir -pv ${TMPROOT-}/usr/share/egl
	cp -fvr usr/share/egl/* ${TMPROOT-}/usr/share/egl/
	cp -fv etc/X11/xorg.conf.jetson_e ${TMPROOT-}/etc/X11/xorg.conf.d/99-tegra.conf
	cp -fv etc/modules ${TMPROOT-}/etc/
	mkdir -p etc/udev/rules.d
	cp -fv etc/udev/rules.d/99-tegra-devices.rules ${TMPROOT-}/etc/udev/rules.d
	cp -fv etc/udev/rules.d/99-tegra-mmc-ra.rules ${TMPROOT-}/etc/udev/rules.d
	INITRD_IMG=$(readlink ${TMPROOT-}/boot/initrd.img)
	cat ${TMPROOT-}/boot/$INITRD_IMG |gunzip > $JETSON_TMP/initrd.cpio
	rm -fr $JETSON_TMP/initrd/
	mkdir -p $JETSON_TMP/initrd/lib/firmware
	cd $JETSON_TMP/initrd
	cat ../initrd.cpio |cpio -id
	cp ${TMPROOT-}/lib/firmware/tegra21x_xusb_firmware lib/firmware
	find * .initrd | cpio -H newc -o > $JETSON_TMP/initrd.cpio
	cd $JETSON_TMP
	gzip -c initrd.cpio > ${TMPROOT-}/boot/$INITRD_IMG
	rm -fr initrd.cpio initrd/
}

wget_jetson_nano() {
	if [ "$NOASK" != 1 ]; then
		rm -fr "$JETSON_PATH/$JETSON_ARCHIVE"
		printn "Warning! $JETSON_ARCHIVE not downloaded!!!" red
		printn "Download? [Yes/No]" bold
		read DOWNLOAD
		case "$DOWNLOAD" in
			[Yy][Ee][Ss])
				;;
			*)
				fatal_error "Error: $JETSON_ARCHIVE not downloaded!!!"
				;;
		esac
	fi
	print_message "Download https://developer.nvidia.com/embedded/$JETSON_ARCHIVE..."
	2>&$LOG_FD1 wget "https://developer.nvidia.com/embedded/dlc/$JETSON_ARCHIVE" \
	  -P "$JETSON_PATH"
	print_done
	print_message "Extract Jetson Nano driver archive..."
	${TAR% *} -xf "$JETSON_PATH/$JETSON_ARCHIVE" -C "$TMP"
	if [ $? -ne 0 ]; then
		rm -fr "$JETSON_PATH/$JETSON_ARCHIVE"
		fatal_error "Error: bad archive. Removed $JETSON_ARCHIVE"
	fi
	print_done
}
