#!/bin/bash
# vim:sw=8:noet

soc_init() {
	MEDIATABLE=msdos
	START_SECTOR=16 # MiB
	FIRMPART_NUM=1
	ROOTPART_NUM=2
	SYSCON=ttyS0,115200n8
}

soc_write_rootfs() {
	if [ -n "${IMAGE-}" ]; then
		# Write image file to the media device
		image_to_media
	else
		# Write rootfs
		[ -z "${IMAGE_OUT-}" ] || prepare_image
		[ -z "${MEDIA-}" ] || prepare_media
		create_disklabel
		create_partition
		[ -z "${IMAGE_OUT-}" ] || finish_partitioning
		sync_partitions
		format_partition
		mount_partition
		write_rootfs
		setup_fstab
		update_cmdline_txt
	fi
}

soc_write_bootloader() {
	[ -n "$TMPROOT" ] || mount_partition
	rm -f "$TMPFIRM"/kernel?.img
	# Writing U-Boot
	print_message "Writing U-Boot..."
	set -e
	if [ "$TARGET" == "rpi_3_b_plus" ]; then
		cp -r "${TMPROOT-}"/usr/share/u-boot/rpi_3/* "$TMPFIRM"/ ||
			fatal_error "failed to write U-Boot"
	fi
	cp -r "${TMPROOT-}"/usr/share/u-boot/$TARGET/* "$TMPFIRM"/ ||
		fatal_error "failed to write U-Boot"
	print_done
}

soc_exit() {
	:
}
